#ifndef _LIBAC_H_
#define _LIBAC_H_


/*************************************************************************
 *
 * 	AVR C library configuration check
 *
 ************************************************************************/

#include <libcfg.h>

#if ! ( defined (AVR_LIBC_1) || defined (AVR_LIBC_2) )
# error "missing parameter: AVR_LIBC_*"
#endif


#include <stddef.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#if defined(AVR_LIBC_1)
#include <io.h>
#include <interrupt.h>
#include <sig-avr.h>
#else
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#endif
#include <inttypes.h>


/*************************************************************************
 *
 * 	DELAY subroutienes (simple and usefull, but inaccurate)
 *
 *************************************************************************
 *
 * BEFORE USING ANY OF THIS STUFF YOU
 * MUST DECLARE THE AMOUNT OF LOOPS
 * WE HAVE TO DO FOR 1ms (DEPAND ON
 * YOUR SYSTEM CLOCK)!
 *
 * EXAMPLE FOR 8MHz:
 *
 * 	CFLAGS += -DDELAY_1MS_COUNT=0x07d0
 *
 * LIST:
 *
 * 	0x07d0 (2000)	@ 8MHz
 * 	0x06d6 (1750)	@ 7MHz
 * 	0x05dc (1500)	@ 6MHz
 * 	0x04e2 (1250)	@ 5MHz
 * 	0x03e8 (1000)	@ 4MHz
 * 	0x02ee (750)	@ 3MHz
 * 	0x01f4 (500)	@ 2MHz
 * 	0x00fa (250)	@ 1MHz
 *
 ************************************************************************/

#ifndef DELAY_1MS_COUNT
# error "missing parameter: DELAY_1MS_COUNT"
# define DELAY_1MS_COUNT 0
#endif

/*
 * Counter loop for N+1 mikroseconds at 8MHz system clock.
 * One zero loop ca. 1s (1s, 125ns jitter).
 *
 * !!! NOT FOR SYSTEM CLOCKS DIFFERENT FROM 8MHz !!!
 */
	void		delay_us	(uint16_t us);

/*
 * Counter loop for N milliseconds depand on system clock.
 * One loop ca. 1ms (0,29% jitter). This function is using
 * the WDR call to avoid watchdog reset.
 *
 * !!! DEPANDS ON DELAY_1MS_COUNT (see above) !!!
 */
	void		delay_ms	(uint16_t ms);

/*
 * even clock delayes:
 */
	void		__delay_9T	(void);
	void		__delay_6T	(void);
	void		__delay_3T	(void);


/*************************************************************************
 *
 * 	LED access control
 *
 ************************************************************************/

	void		led_init	(void);
	void		led_put		(uint8_t bitset_level);

#if defined(AVR_LIBC_1)
#define			led_cls()	outp(0, (LED_PORT))
#else
#define			led_cls()	LED_PORT = 0
#endif
#define			led_clear()	led_cls()

	void		led_graph_init	(int16_t led_maxlevel);
	void		led_graph_put	(int16_t bitset_level);

/* led_options:
 *
 * Bit:   7 6 5 4   3 2 1   0
 *        | | | |   | | |   \___ fill mode (bar or point graph)
 *        | | | |   | | |           0 point graph
 *        | | | |   | | |           1 bar graph
 *        | | | |   \_\_\_______ margin bit position
 *        | | | |                 000 Bit 0 (bottom)
 *        | | | |                 100 Bit 4 (middle)
 *        | | | |                 111 Bit 7 (top)
 */
#define			LED_FILL	(BV(0))
#define			LED_MARGIN(M)	((M) << 1)

	void		led_graph_control(int16_t led_maxlevel,
					  int8_t  led_options);


/*************************************************************************
 *
 * 	KEY access control
 *
 *************************************************************************
 *
 * key_scancode:
 *
 * 	Bit:   7 6 5 4 3 2 1 0
 *	       | | | | | | | \___ not used
 *	       | | | | | | \_____ not used
 *	       | | | | | \_______ not used
 *	       | | | | \_________ T5
 *	       | | | \___________ T4
 *	       | | \_____________ T3
 *	       | \_______________ T2
 *	       \_________________ T1
 *
 *
 * AVR-Ctrl key numbers:
 *		_______________________________
 *	       |                               |
 *	       |      LCD ...                  |
 *	       |_______________________________|
 *		 _____ _____ _____ _____ _____
 *		|  T1 |  T2 |  T3 |  T4 |  T5 |
 *		|_____|_____|_____|_____|_____|
 *		|\___/|\___/|\___/|\___/|\___/|
 *		||___|||___|||___|||___|||___||
 *		|/___\|/___\|/___\|/___\|/___\|
 *
 ************************************************************************/

	void		key_init	(void);
#if defined(AVR_LIBC_1)
#define			key_scancode()	(inp(KEY_PORT_IN) & KEY_ALL)
#else
#define			key_scancode()	(KEY_PORT_IN & KEY_ALL)
#endif

#define			KEY_T1		BV(7)
#define			KEY_T2		BV(6)
#define			KEY_T3		BV(5)
#define			KEY_T4		BV(4)
#define			KEY_T5		BV(3)
#define			KEY_ALL		( KEY_T1	\
					| KEY_T2	\
					| KEY_T3	\
					| KEY_T4	\
					| KEY_T5 )


/*************************************************************************
 *
 * 	KEY access control
 *
 *************************************************************************
 *
 *   Program:    LCD driver header
 *   Created:    27.8.99 20:26
 *   Author:     Brajer Vlado
 *                 vlado.brajer@kks.s-net.net
 *   Comments:   functions & definitions
 *
 *   Remarks:    20020513 Stephan Linz <linz@li-pro.net>
 *               adapted to AVR-Ctrl board designed by
 *               http://www.mikrocontroller.com
 *
 *************************************************************************
 *
 * lcd_matrix:
 *
 * 	Bit:   7 6   5 4 3 2 1 0
 *	       | |   \_\_\_\_\_\___ numbers of columns each row
 *	       \_\_________________ numbers of rows:
 *				      00 same as 11 (compatibillity)
 *				      01 one row
 *				      10 two rows
 *				      11 four rows
 *
 ************************************************************************/

#if ! ( defined(LCD_COM_POLLING) || defined(LCD_COM_DELAY) )
# error "missing parameter (LCD_COM_POLLING or LCD_COM_DELAY)"
#endif

	void		lcd_init	(uint8_t lcd_matrix);

/* Brajer Vlado conformity */
	void		lcd_cls		(void);
	void		lcd_home	(void);
	void		lcd_goto	(uint8_t addr);
	void		lcd_control	(uint8_t disonoff,
					 uint8_t curonoff,
					 uint8_t curblink);
	void		lcd_putch	(uint8_t data);
	void		lcd_putstr	(uint8_t *data);
	void		lcd_printbin	(uint8_t x);
	void		lcd_printhex	(uint8_t i);
	void		lcd_print2	(uint16_t x);
	void		lcd_print3	(uint16_t x);
	void		lcd_print5	(uint16_t x);
	void		lcd_print10	(uint32_t x);
	int		lcd_printf	(const char *format, ...);

/* CodeVision conformity */
	void		lcd_gotoxy	(uint8_t x,
					 uint8_t y);
#define			lcd_clear()	lcd_cls()
#define			lcd_putchar(C)	lcd_putch((uint8_t)(C))
#define			lcd_puts(STR)	lcd_putstr((uint8_t*)(STR))
	void		lcd_write_byte	(uint8_t addr,
					 uint8_t data);
	void		_lcd_write_data	(uint8_t data);
	void		_lcd_ready	(void);

// not yet implemented in this library:
// unsigned char lcd_read_byte(unsigned char addr);
// void lcd_putsf(STR);


/*************************************************************************
 *
 * 	ONE WIRE device access control
 *
 *************************************************************************
 *
 * You have to set OW_DEVICES to a value greater than 0. If OW_DEVICES
 * equal one (1), so you have to connect one and only one 1-Wire device
 * to the bus, because there is no further ROM code check inside of the
 * library while access to the bus (then we use SKIP ROM and no search
 * commands).
 *
 * More than one device will be handeld if you set OW_DEVICES to a value
 * greater than one. There is no upper limit only SRAM space :-)
 *
 ************************************************************************/

#ifndef OW_DEVICES
# error "missing parameter: OW_DEVICES"
# define OW_DEVICES 0
#endif

#ifdef _LIBAC_OW_C_
#define extern
#endif

struct ow_rom_code_s {
	uint8_t			family;
	uint8_t			serial[6];
	uint8_t			crc;
};
typedef struct ow_rom_code_s	ow_rom_code_st;

struct ow_device_s {
	ow_rom_code_st		rom_code;
	union {
	  uint8_t		status;		// status byte (e.g. DS2405)
	  uint8_t		flags;		// device flags with:
	  struct {
	    unsigned		  alarm:1;	// alarm
	    unsigned		  parpower:1;	// parasite power supply
	    //unsigned		  unused:1;
	    //unsigned		  unused:1;
	    //unsigned		  unused:1;
	    //unsigned		  unused:1;
	    //unsigned		  unused:1;
	    //unsigned		  unused:1;
	  };
	};
};
typedef struct ow_device_s	ow_device_st;

extern	ow_device_st	ow_devices[OW_DEVICES];
extern	uint8_t		ow_devices_active;

	uint8_t		ow_init		(void);
	uint8_t		ow_rom_search	(void);
	uint8_t		ow_byte_read	(void);
	uint8_t		ow_byte_write	(uint8_t byte);
	uint8_t		ow_crc8		(uint8_t *data,
					 size_t  length);
	uint8_t		ow_function	(uint8_t code,
					 uint8_t device,
					 uint8_t family);
	void		ow_ready	(void);
	uint8_t		ow_buf_read	(uint8_t *buf,
					 size_t  buf_size,
					 uint8_t crc_included);
	uint8_t		ow_buf_write	(uint8_t *buf,
					 size_t  buf_size,
					 uint8_t crc_included);

/* CodeVision conformity */
#define			w1_init()	ow_init()
#define			w1_read()	(unsigned char)ow_byte_read()
#define			w1_write(D)	ow_byte_write((uint8_t)(D))
#define			w1_crc8(P,N)	(unsigned char)ow_crc8( \
						(uint8_t *)(P), \
						(int)(N))
#define			w1_search(C,P)	(unsigned char)ow_rom_search()

// not yet full implemented in this library:
// unsigned char w1_search(unsigned char cmd, void *p);

#ifdef _LIBAC_OW_C_
#undef extern
#endif


/*************************************************************************
 *
 * 	DS1820 temperature sensor access control
 *
 ************************************************************************/

#define			DS1820_FAMILY	0x10

	uint8_t		ds1820_convert	(uint8_t device);
	int16_t		ds1820_temp10_C	(uint8_t device);

/* CodeVision conformity */
	int		ds1820_temperature_10(unsigned char *addr);

#endif /* _LIBAC_H_ */
